const displayWaitingSpinner = require('./waiting-spinner');
const displayErrorMessage = require('./display-error-message');
const lang = require('./mailytica-lang');
let categoryName,
    responseButtonText,
    changeSalutationButton
anredeSmartResponse = '';

lang(function (languageFile) {
    categoryName = i18n(languageFile[0].category);
    responseButtonText = i18n(languageFile[0].response);
    changeSalutationButton = i18n(languageFile[0].salutation);
});

let client = ZAFClient.init(),
    retrievedSmartResponseProposals,
    currentSmartResponse,
    topicModelUrl = 'https://api.mailytica.ai/api';
tokenFromZendesk = '',
    httpRequestTimeout = 30000;

/*Handlebars.registerHelper('percentage', function(distance) {
  return distance.toFixed(2) * 100 + ' %';
});

Handlebars.registerHelper("log", function(logging) {
  console.log('handlebars log', logging);
});*/

Handlebars.registerHelper("increase", function (value, options) {
    return parseInt(value) + 1;
});

Handlebars.registerHelper('json', function (context) {
    return JSON.stringify(context);
});

client.invoke('resize', {width: '100%', height: '570px'});

client
    .metadata()
    .then(function (metadata) {
        httpRequestTimeout = metadata.settings.httpRequestTimeout;
        tokenFromZendesk = metadata.settings.apiToken;
    });

loadNewestTicketComment();

function getSettings(url, message) {
    return {
        url: topicModelUrl + url,
        headers: {
            'Authorization': 'Bearer ' + tokenFromZendesk
        },
        secure: true,
        type: 'POST',
        contentType: 'application/json',
        data: JSON.stringify(message),
        httpCompleteResponse: true,
        timeout: httpRequestTimeout
    }
}

function getDeleteRequest(url) {
    return {
        url: topicModelUrl + url,
        headers: {
            'Authorization': 'Bearer ' + tokenFromZendesk
        },
        secure: true,
        type: 'DELETE',
        httpCompleteResponse: true,
        timeout: httpRequestTimeout
    }
}

function getCommentDatetime(comment) {

    return comment.created_at;
}

function loadNewestTicketComment() {

    displayWaitingSpinner(true);

    client
        .get('ticket')
        .then(function (data) {

            let ticket = data.ticket;

            let ticketId = ticket.id;

            let inboundComments = ticket
                .comments
                .filter(function (comment) {
                    return comment.author.role === "end-user"
                });

            if (inboundComments.length === 0) { // new ticket

                currentMessage = {
                    messageId: '',
                    subject: '',
                    sender: '',
                    recipients: [],
                    message: '',
                    send: '',
                    metadataFields: []
                };

                requestSmartResponseProposal(currentMessage);

                return currentMessage;

            } else {

                let newestInboundCommentId = inboundComments[0].id;

                let newestInboundCommentPromise = getNewestInboundComment(ticketId, newestInboundCommentId);

                newestInboundCommentPromise.then(function (newestInboundComment) {

                    let commentId = newestInboundComment.id,
                        htmlBody = newestInboundComment.html_body,
                        messageId = ticketId + '_' + commentId,
                        subject = ticket.subject,
                        sender = getSender(newestInboundComment),
                        recipient = getRecipient(newestInboundComment),
                        commentDatetime = getCommentDatetime(newestInboundComment),
                        metadataFieldsPromise = requestTicketFields();

                    metadataFieldsPromise.then(function (metadataFields) {

                        currentMessage = {
                            messageId: messageId,
                            subject: subject,
                            sender: sender,
                            recipients: [recipient],
                            message: htmlBody,
                            send: commentDatetime,
                            metadataFields: metadataFields
                        };

                        requestSmartResponseProposal(currentMessage);

                        return currentMessage;

                    });
                });
            }
        })
}

function requestSmartResponseProposal(message) {

    // https://develop.zendesk.com/hc/en-us/articles/360001074948-Zendesk-Apps-tutorial-Sending-data-to-an-external-application
    // https://developer.zendesk.com/apps/docs/core-api/client_api#client.requestoptions
    // https://develop.zendesk.com/hc/en-us/articles/360002034827-How-Apps-framework-client-request-works

    let settings = getSettings(
        '/classifyMessage',
        message
    );

    settings['success'] = function (data) {
        smartResponseProposal(data, 0);
    };

    settings['error'] = function (error) {
        displayWaitingSpinner(false);
        displayErrorMessage(error);
    };

    $.ajax(settings);
}

function getNewestInboundComment(ticketId, commentId) {

    return client.request({
        url: '/api/v2/tickets/' + ticketId + '/comments.json',
        httpCompleteResponse: true
    })
        .then(
            function (response) {
                let comments = response
                    .responseJSON
                    .comments;

                let filteredComments = comments
                    .filter(function (comment) {
                        return comment.id === commentId;
                    });

                if (filteredComments.length === 0) {

                    return comments[0];
                } else {

                    return filteredComments[0];
                }

                return comment;
            }
        );
}

function getRecipient(comment) {

    let name = comment.via.source.to.name;
    let email = comment.via.source.to.address;

    if (name === undefined && email === undefined) {
        return "";
    } else if (name === undefined && email !== undefined) {
        return email;
    } else {
        return name + " <" + email + ">";
    }
}

function getSender(comment) {

    let name = comment.via.source.from.name;
    let email = comment.via.source.from.address;

    if (name === undefined && email === undefined) {
        return "";
    } else if (name === undefined && email !== undefined) {
        return email;
    } else {
        return name + " <" + email + ">";
    }
}

function requestCustomTicketFieldValue(customTicketFieldName) {

    var requestName = 'ticket.customField:' + customTicketFieldName;

    return client
        .get(requestName)
        .then(function (data) {
            let customTicketFieldValue = data[requestName];

            return customTicketFieldValue;
        });
}

function requestTicketFields() {

    return client
        .get('ticketFields')
        .then(function (data) {

            var customTicketFieldsPromises = data
                .ticketFields
                .filter(function (ticketField) {
                    return ticketField.name.startsWith("custom_field");
                })
                .map(function (customTicketField) {

                    return requestCustomTicketFieldValue(customTicketField.name)
                        .then(function (customTicketFieldValue) {

                            var metadata = {
                                metadataFieldId: customTicketField.label,
                                metadataValue: customTicketFieldValue
                            };

                            return metadata;
                        });
                });

            var customTicketFieldsPromise = Promise
                .all(customTicketFieldsPromises)
                .then(function (customTicketFields) {
                    return customTicketFields.filter(function (customTicketField) {
                        return customTicketField.metadataValue !== null;
                    });
                });

            return customTicketFieldsPromise;
        });
}

function smartResponseProposal(serverResponse, smartResponseId) {

    this.retrievedSmartResponseProposals = serverResponse;

    let smartResponseProposals = serverResponse
        .smartResponseProposals
        .map(function (smartResponseProposal, index) {

            let note = smartResponseProposal.label;
            if (smartResponseProposal.note.length > 0) {
                note = smartResponseProposal.note;
            }

            return {
                key: (index + 1),
                index: index,
                isCurrent: smartResponseProposal.id === smartResponseId,
                id: smartResponseProposal.id,
                responseLabel: smartResponseProposal.topicLabel,
                smartResponseLabel: smartResponseProposal.label,
                smartResponse: smartResponseProposal.smartResponse,
                smartResponseText: smartResponseProposal.smartResponseText,
                note: note
            };
        });

    this.currentSmartResponse = smartResponseProposals.filter(smartResponseProposal => smartResponseProposal.isCurrent)[0];
    if (this.currentSmartResponse === undefined) {
        this.currentSmartResponse = smartResponseProposals[0];
    }

    let currentTopicLabel = this.currentSmartResponse.responseLabel;

    // Create an object for storing unique category names
    let uniqueCategoriesMap = new Set();
    let uniqueCategories = [];
    smartResponseProposals.forEach(function (topic) {
        if (!uniqueCategoriesMap.has(topic.responseLabel)) {
            uniqueCategoriesMap.add(topic.responseLabel);
            uniqueCategories.push(topic);
        }
    });

    // Generate new category groups to define which category tab is active
    let categoryGroups = uniqueCategories.map(function (topic) {

        let smartResponsesForTopic = smartResponseProposals.filter(smartResponse => smartResponse.responseLabel === topic.responseLabel);

        let categoryGroup = {
            responseLabel: topic.responseLabel,
            isCurrent: topic.responseLabel === currentTopicLabel,
            index: topic.index,
            hasChildren: smartResponsesForTopic.length > 1,
            smartResponses: smartResponsesForTopic
        };

        return categoryGroup;
    });

    let categories = serverResponse
        .topics
        .map(function (category) {
            return {
                category: category,
                selected: category === serverResponse.classification.label
            };
        });

    // Push a standard category into the category's list
    categories.push({
        category: 'Standard',
        selected: 'Anrede' === serverResponse.classification.label || 'Standard' === serverResponse.classification.label
    });

    categories.sort((a, b) => (a.category > b.category) ? 1 : ((b.category > a.category) ? -1 : 0));

    let confidenceColour = '#78A100';

    if (serverResponse.classification.confidence < 0.80) {
        confidenceColour = '#EB6651';
    } else if (serverResponse.classification.confidence < 0.85) {
        confidenceColour = '#EFC93D';
    } else if (serverResponse.classification.confidence < 0.90) {
        confidenceColour = '#F79A3E';
    }

    this.anredeSmartResponse = serverResponse
        .smartResponseProposals
        .filter(function (proposal) {
            return proposal.topicLabel === 'Anrede';
        })[0];

    let note = this.currentSmartResponse.label;
    if (this.currentSmartResponse.note.length > 0) {
        note = this.currentSmartResponse.note;
    }

    let smartResponseTextSalutation = $(anredeSmartResponse.smartResponse).text(),
        srpHandlebar = {
            classification: serverResponse.classification,
            confidence: serverResponse.classification.confidence,
            confidenceColour: confidenceColour,
            smartResponse: this.currentSmartResponse.smartResponse,
            note: note,
            smartResponseTextSalutation: smartResponseTextSalutation,
            categories: categories,
            categoryName: categoryName,
            responseButtonText: responseButtonText,
            changeSalutationButton: changeSalutationButton,
            categoryGroups: categoryGroups
        },
        source = $('#smart-response-template').html(),
        template = Handlebars.compile(source),
        html = template(srpHandlebar);

    $('#handlebarContent').html(html);

    // Load select2 library after handlebar is fully loaded
    $('#categoriesSelect').select2();

    // Apply active class to the tab
    $('.label-tab[aria-selected]').each(function () {
        let thisLabel = this;
        let tabVisible = $(thisLabel).attr('aria-selected');
        if (tabVisible === 'true') {
            $(thisLabel).addClass('active');
        } else {
            $(thisLabel).removeClass('active');
        }
    });

    // Arrow navigating for category tab
    $('#slideRight').click(function () {
        $('#nav-container').animate({scrollLeft: '+=200'}, 500, 'swing');
    });

    $('#slideLeft').click(function () {
        $('#nav-container').animate({scrollLeft: '-=200'}, 500, 'swing');
    });

    // If category tabs doesnt overflow, hide the scrollbar
    setTimeout(function () {
        if ($('.nav').width() < 320) {
            $('#nav-container').addClass('not-overflow');
            $('.nav').addClass('fullwidth');
            $('.disable-button').show();
            $('#slideLeft').hide();
            $('#slideRight').hide();
        } else {
            $('#nav-container').removeClass('not-overflow');
            $('.nav').removeClass('fullwidth');
            $('.disable-button').hide();
            $('#slideLeft').show();
            $('#slideRight').show();
        }
    }, 0);

    $('#categoriesSelect').change(function () {
        updateCategory(this.value);
    });

    displayWaitingSpinner(false);
}

function addClassificationTag(classification) {

    if (classification.toLowerCase() !== 'standard' && classification.toLowerCase() !== 'anrede') {

        client.invoke('ticket.tags.add', classification.replaceAll(" ", "_"));
    }
}

function displayDraftResponse() {
    displayWaitingSpinner(true);

    setTimeout(function () {
        draftSmartResponse();
        displayWaitingSpinner(false);
    }, 100);
}

function draftSmartResponse() {

    let currentText = client.get('comment')

    currentText.then(function (data) {

        let text = data.comment.text;

        if (text === '<p></p>' || text.trim() === '') {
            client.invoke('comment.appendHtml', this.currentSmartResponse.smartResponse);
        } else {
            client.invoke('comment.appendHtml', this.currentSmartResponse.smartResponseText);
        }
    });

    addClassificationTag(this.currentSmartResponse.responseLabel);

    consumeSmartResponse(this.currentMessage.messageId, this.currentSmartResponse.id);
}

function smartResponseById(id) {
    displayWaitingSpinner(true);
    setTimeout(function () {
        smartResponseProposal(this.retrievedSmartResponseProposals, id);
        displayWaitingSpinner(false);
    }, 100);
}

function smartResponseByTopic(topic) {
    displayWaitingSpinner(true);
    setTimeout(function () {

        let smartResponseId = this.retrievedSmartResponseProposals
            .smartResponseProposals
            .filter(smartResponseProposal => smartResponseProposal.topicLabel === topic)
            .map(smartResponseProposal => smartResponseProposal.id)[0];

        smartResponseProposal(this.retrievedSmartResponseProposals, smartResponseId);
        displayWaitingSpinner(false);
    }, 100);
}

function updateCategory(goldStandardLabel) {
    postGoldStandardClassification(currentMessage, goldStandardLabel);
}

function showSalutationSetting() {
    $('#edit-salutation-container').removeClass('is-hidden');

    let oldSalutation = $('#new-salutation').val();

    document.onkeyup = function (e) {
        if (e.key === 'Enter') {
            let newSalutation = $('#new-salutation').val();
            changeSalutation(currentMessage, newSalutation);
        }
        if (e.key === 'Escape') {
            $('#edit-salutation-container').addClass('is-hidden');
        }
    };


    $('#save').click(function () {
        let newSalutation = $('#new-salutation').val();

        if (newSalutation === oldSalutation) {
            $('#edit-salutation-container').addClass('is-hidden');
        } else {
            changeSalutation(currentMessage, newSalutation);
        }
    });
}

function closeSalutation() {
    $('#edit-salutation-container').addClass('is-hidden');
}

function changeSalutation(message, salutation) {
    displayWaitingSpinner(true);
    let changeSalutationCommand = {
            message: message,
            salutation: salutation
        },
        settings = getSettings(
            '/persistCustomerSalutation',
            changeSalutationCommand
        );

    settings['success'] = function (data) {
        let oldCategory = $('#categoriesSelect').val();
        smartResponseProposal(data, 0);
        postGoldStandardClassification(message, oldCategory);
        displayWaitingSpinner(false);
    };

    settings['error'] = function (errorMessage) {
        displayErrorMessage(errorMessage);
    };

    $.ajax(settings);
}

function postGoldStandardClassification(message, goldStandardLabel) {

    displayWaitingSpinner(true);

    if (goldStandardLabel === 'Standard') {
        let messageID = message.messageId,
            settings = getDeleteRequest(
                '/message/' + messageID
            );

        settings['success'] = function () {
            standardCategoryGoldStandardClassificationCall(message, 'Standard')
        };

        settings['error'] = function (errorMessage) {
            displayErrorMessage(errorMessage);
        };

        $.ajax(settings);
    } else {
        let classifyGoldStandardCommand = {
                message: message,
                goldStandardLabel: goldStandardLabel
            },
            settings = getSettings(
                '/goldStandardClassification',
                classifyGoldStandardCommand
            );

        settings['success'] = function (data) {
            smartResponseProposal(data, 0);
            displayWaitingSpinner(false);
        };

        settings['error'] = function (errorMessage) {
            displayErrorMessage(errorMessage);
        };

        $.ajax(settings);
    }
}

function standardCategoryGoldStandardClassificationCall(message, goldStandardLabel) {
    let classifyGoldStandardCommand = {
            message: message,
            goldStandardLabel: goldStandardLabel
        },
        settings = getSettings(
            '/goldStandardClassification',
            classifyGoldStandardCommand
        );

    settings['success'] = function (data) {
        smartResponseProposal(data, 0);
        displayWaitingSpinner(false);
    };

    settings['error'] = function (errorMessage) {
        displayErrorMessage(errorMessage);
    };

    $.ajax(settings);
}

function consumeSmartResponse(emailId, smartResponseProposalId) {
    let consumeSmartResponseProposalCommand = {
            emailId: emailId,
            smartResponseProposalId: smartResponseProposalId
        },
        settings = getSettings(
            '/consumeSmartResponse',
            consumeSmartResponseProposalCommand
        );

    settings['success'] = function (data) {

    };

    settings['error'] = function (errorMessage) {
        console.error('error:' + errorMessage);
    };

    $.ajax(settings);
}

module.exports = {
    draftSmartResponse: draftSmartResponse,
    displayWaitingSpinner: displayWaitingSpinner,
    loadNewestTicketComment: loadNewestTicketComment,
    smartResponseById: smartResponseById,
    smartResponseByTopic: smartResponseByTopic,
    showSalutationSetting: showSalutationSetting,
    closeSalutation: closeSalutation,
    displayDraftResponse: displayDraftResponse
};
